==> copilot-24.3.3-sources.jar: com/vaadin/copilot/ProjectFileHandler.java <==
package com.vaadin.copilot;

import com.vaadin.base.devserver.DevToolsInterface;
import elemental.json.Json;
import elemental.json.JsonObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;

public class ProjectFileHandler implements CopilotCommand {

    private final ProjectManager projectManager;

    public ProjectFileHandler(ProjectManager projectManager) {
        this.projectManager = projectManager;
    }

    @Override
    public boolean handleMessage(String command, JsonObject data,
            DevToolsInterface devToolsInterface) {
        if (command.equals("get-project-file")) {
            String filename = data.getString("filename");
            String reqId = data.getString(KEY_REQ_ID);

            JsonObject responseData = Json.createObject();
            responseData.put(KEY_REQ_ID, reqId);
            try {
                responseData.put("content", projectManager.readFile(filename));
                devToolsInterface.send(Copilot.PREFIX + "project-file",
                        responseData);
            } catch (IOException e) {
                responseData.put("error", true);
                devToolsInterface.send(Copilot.PREFIX + "project-file",
                        responseData);
                getLogger().error("Unable to read project file {}", filename,
                        e);
            }
            return true;
        } else if (command.equals("write-project-file")) {
            String filename = data.getString("filename");
            String reqId = data.getString(KEY_REQ_ID);
            String content = data.getString("content");
            JsonObject responseData = Json.createObject();
            responseData.put(KEY_REQ_ID, reqId);
            try {
                projectManager.writeFile(filename, content);
                devToolsInterface.send(Copilot.PREFIX + "project-file-written",
                        responseData);
            } catch (IOException e) {
                responseData.put("error", true);
                devToolsInterface.send(Copilot.PREFIX + "project-file-written",
                        responseData);
                getLogger().error("Unable to write project file {}", filename,
                        e);
            }
            return true;
        }

        return false;
    }

    private Logger getLogger() {
        return LoggerFactory.getLogger(getClass());
    }
}

==> copilot-24.4.6-sources.jar: com/vaadin/copilot/ProjectFileHandler.java <==
package com.vaadin.copilot;

import com.vaadin.base.devserver.DevToolsInterface;
import com.vaadin.copilot.ide.CopilotIDEPlugin;
import elemental.json.Json;
import elemental.json.JsonObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.io.IOException;

public class ProjectFileHandler implements CopilotCommand {

    private static final String COPILOT_UNDO_LABEL = CopilotIDEPlugin
            .undoLabel("File Update");

    private final ProjectManager projectManager;

    public ProjectFileHandler(ProjectManager projectManager) {
        this.projectManager = projectManager;
    }

    @Override
    public boolean handleMessage(String command, JsonObject data,
            DevToolsInterface devToolsInterface) {
        if (command.equals("get-project-file")) {
            String filename = data.getString("filename");
            String reqId = data.getString(KEY_REQ_ID);

            JsonObject responseData = Json.createObject();
            responseData.put(KEY_REQ_ID, reqId);
            try {
                responseData.put("content", projectManager.readFile(filename));
                devToolsInterface.send(Copilot.PREFIX + "project-file",
                        responseData);
            } catch (IOException e) {
                getLogger().debug("Unable to read project file {}", filename,
                        e);
                ErrorHandler.sendErrorResponse(devToolsInterface, command,
                        responseData, "Unable to read file", e);
            }
            return true;
        } else if (command.equals("write-project-file")) {
            String filename = data.getString("filename");
            String reqId = data.getString(KEY_REQ_ID);
            String content = data.getString("content");
            JsonObject responseData = Json.createObject();
            responseData.put(KEY_REQ_ID, reqId);
            try {
                projectManager.writeFile(filename, COPILOT_UNDO_LABEL, content);
                devToolsInterface.send(Copilot.PREFIX + "project-file-written",
                        responseData);
            } catch (IOException e) {
                getLogger().debug("Unable to write project file {}", filename,
                        e);
                ErrorHandler.sendErrorResponse(devToolsInterface, command,
                        responseData, "Unable to write file", e);
            }
            return true;
        } else if (command.equals("write-project-file-binary")) {
            JsonObject responseData = Json.createObject();
            responseData.put(KEY_REQ_ID, data.getString(KEY_REQ_ID));

            String filename = data.getString("filename");
            String base64 = data.getString("base64Content");
            boolean renameIfExists = data.getBoolean("renameIfExists");
            try {
                File written = projectManager.writeFileBase64(filename,
                        COPILOT_UNDO_LABEL, base64, renameIfExists);
                responseData.put("filename", written.getName());
                devToolsInterface.send(Copilot.PREFIX + command + "-response",
                        responseData);
            } catch (IOException e) {
                responseData.put("error", true);
                devToolsInterface.send(Copilot.PREFIX + "project-file-written",
                        responseData);
                getLogger().error("Unable to write project file {}", filename,
                        e);
            }
            return true;
        }

        return false;
    }

    private Logger getLogger() {
        return LoggerFactory.getLogger(getClass());
    }
}
